import { PRE_GAME, PRE_ROUND, IN_ROUND, POST_ROUND, POST_GAME } from "./GameState";
import { createStore } from "../store";
import { createGame, joinGame, sendGuess } from "./apiMethods";

export const [
  {
    useGameId,
    usePlayerName,
    useLastRound,
    useGameJoined,
    useGameState,
  },
  dispatch,
] = createStore({
  gameId: null,
  playerName: null,
  lastRound: {
    roundNum: -1,
    targetPoint: {
      lat: 0,
      lng: 0,
    },
    score: -1,
    totalScore: -1,
  },
  gameJoined: false,
  gameState: PRE_GAME,
}, {
  setPlayerName: ([set], playerName) => set({ playerName }),
  goToLobby: ([set], gameId) => set({ gameId, gameState: PRE_ROUND }),
  createGame: async ([set, get], timer) => {
    const name = get.playerName();
    const gameId = await createGame(name, timer);
    set({ gameId, gameJoined: true, gameState: PRE_ROUND });
  },
  joinGame: async ([set, get]) => {
    const gameId = get.gameId();
    const name = get.playerName();
    await joinGame(gameId, name);
    set({ gameJoined: true });
  },
  startRound: ([set]) => set({ gameState: IN_ROUND }),
  submitGuess: async ([set, get], selectedPoint, roundNum, targetPoint) => {
    const gameId = get.gameId();
    const name = get.playerName();
    const { score, totalScore } = await sendGuess(
      gameId,
      name,
      roundNum,
      selectedPoint || { timeout: true }
    );
    set({
      lastRound: {
        roundNum,
        targetPoint,
        score,
        totalScore,
      },
      gameState: POST_ROUND,
    });
  },
  goToSummary: ([set, get], gameId) => set({
    gameId: gameId || get.gameId(),
    gameState: POST_GAME,
  }),
  resetGame: ([set]) => set({
    gameJoined: false,
    gameState: PRE_GAME,
  }),
});